export default function taskManager(tasks: Array<() => Promise<void>>, maxConcurrentTasks: number): Promise<any[]> {
  return new Promise((resolve, reject) => {
    const returnValues = Array(tasks.length);
    const remainingTasks = tasks;
    let currentlyRunningTasks = 0;

    const startNewTask = () => {
      //Exit if we completed all tasks
      if (remainingTasks.length === 0) {
        if (currentlyRunningTasks === 0) {
          return resolve(returnValues);
        }
      } else {
        //If there is at least one task left, complete it
        const curTask = remainingTasks.pop() as () => Promise<void>;
        const curTaskIndex = remainingTasks.length;
        currentlyRunningTasks += 1;
        curTask().then((...result) => {
          returnValues[curTaskIndex] = result;
          currentlyRunningTasks -= 1;
          return startNewTask();
        }).catch((error) => {
          reject(error);
        });
      }
    };

    for (let i = 0; i < maxConcurrentTasks; i += 1) {
      startNewTask();
    }
  });
}
